@@ -216,11 +216,12 @@ typedef struct {
   const GVariantType *type;
 } VariantKeyfileMapping;
 
-static const VariantKeyfileMapping drive_configuration_mapping[4] = {
-  {"ata-pm-standby",          "ATA", "StandbyTimeout",    G_VARIANT_TYPE_INT32},
-  {"ata-apm-level",           "ATA", "APMLevel",          G_VARIANT_TYPE_INT32},
-  {"ata-aam-level",           "ATA", "AAMLevel",          G_VARIANT_TYPE_INT32},
-  {"ata-write-cache-enabled", "ATA", "WriteCacheEnabled", G_VARIANT_TYPE_BOOLEAN},
+static const VariantKeyfileMapping drive_configuration_mapping[5] = {
+  {"ata-pm-standby",             "ATA", "StandbyTimeout",       G_VARIANT_TYPE_INT32},
+  {"ata-apm-level",              "ATA", "APMLevel",             G_VARIANT_TYPE_INT32},
+  {"ata-aam-level",              "ATA", "AAMLevel",             G_VARIANT_TYPE_INT32},
+  {"ata-write-cache-enabled",    "ATA", "WriteCacheEnabled",    G_VARIANT_TYPE_BOOLEAN},
+  {"ata-read-lookahead-enabled", "ATA", "ReadLookaheadEnabled", G_VARIANT_TYPE_BOOLEAN},
 };
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -958,7 +959,6 @@ handle_eject (UDisksDrive           *_drive,
   gchar *escaped_device = NULL;
   uid_t caller_uid;
   gid_t caller_gid;
-  pid_t caller_pid;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -987,18 +987,6 @@ handle_eject (UDisksDrive           *_drive,
       goto out;
     }
 
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_pid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_error_free (error);
-      goto out;
-    }
-
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
@@ -1025,7 +1013,7 @@ handle_eject (UDisksDrive           *_drive,
     {
       action_id = "org.freedesktop.udisks2.eject-media-system";
     }
-  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
+  else if (!udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
     {
       action_id = "org.freedesktop.udisks2.eject-media-other-seat";
     }
@@ -1180,7 +1168,7 @@ handle_set_configuration (UDisksDrive           *_drive,
                                              data,
                                              data_len,
                                              0600, /* mode to use if non-existant */
-                                             &error) != 0)
+                                             &error))
     {
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -1284,7 +1272,7 @@ send_scsi_command_sync (gint      fd,
         {
           g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                        "Non-GOOD SCSI status from SGIO v4 ioctl: "
-                       "device_status=%d transport_status=%d driver_status=%d",
+                       "device_status=%u transport_status=%u driver_status=%u",
                        io_v4.device_status,
                        io_v4.transport_status,
                        io_v4.driver_status);
@@ -1299,8 +1287,22 @@ send_scsi_command_sync (gint      fd,
 }
 
 static gboolean
-send_scsi_start_stop_command_sync (gint      fd,
-                                   GError  **error)
+send_scsi_synchronize_cache_command_sync (gint      fd,
+                                          GError  **error)
+{
+  uint8_t cdb[10];
+
+  /* SBC3 (SCSI Block Commands), 5.18 SYNCHRONIZE CACHE (10) command
+   */
+  memset (cdb, 0, sizeof cdb);
+  cdb[0] = 0x35;                        /* OPERATION CODE: SYNCHRONIZE CACHE (10) */
+
+  return send_scsi_command_sync (fd, cdb, sizeof cdb, error);
+}
+
+static gboolean
+send_scsi_start_stop_unit_command_sync (gint      fd,
+                                        GError  **error)
 {
   uint8_t cdb[6];
 
@@ -1336,7 +1338,6 @@ handle_power_off (UDisksDrive           *_drive,
   gchar *escaped_device = NULL;
   uid_t caller_uid;
   gid_t caller_gid;
-  pid_t caller_pid;
   GList *sibling_objects = NULL, *l;
   gint fd = -1;
 
@@ -1390,18 +1391,6 @@ handle_power_off (UDisksDrive           *_drive,
         }
     }
 
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_pid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_error_free (error);
-      goto out;
-    }
-
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
@@ -1428,7 +1417,7 @@ handle_power_off (UDisksDrive           *_drive,
     {
       action_id = "org.freedesktop.udisks2.power-off-drive-system";
     }
-  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
+  else if (!udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
     {
       action_id = "org.freedesktop.udisks2.power-off-drive-other-seat";
     }
@@ -1479,10 +1468,11 @@ handle_power_off (UDisksDrive           *_drive,
         }
     }
 
-  /* Send the "SCSI START STOP UNIT" command to request that the unit
-   * be stopped but don't treat failure as fatal. In fact some
-   * USB-attached hard-disks fails with this command, probably due to
-   * the SCSI/SATA translation layer.
+  /* Send the "SCSI SYNCHRONIZE CACHE" and then the "SCSI START STOP
+   * UNIT" command to request that the unit be stopped. Don't treat
+   * failures as fatal. In fact some USB-attached hard-disks fails
+   * with one or both of these commands, probably due to the SCSI/SATA
+   * translation layer.
    */
   fd = open (udisks_block_get_device (block), O_RDONLY|O_NONBLOCK|O_EXCL);
   if (fd == -1)
@@ -1494,7 +1484,21 @@ handle_power_off (UDisksDrive           *_drive,
                                              udisks_block_get_device (block));
       goto out;
     }
-  if (!send_scsi_start_stop_command_sync (fd, &error))
+
+  if (!send_scsi_synchronize_cache_command_sync (fd, &error))
+    {
+      udisks_warning ("Ignoring SCSI command SYNCHRONIZE CACHE failure (%s) on %s",
+                      error->message,
+                      udisks_block_get_device (block));
+      g_clear_error (&error);
+    }
+  else
+    {
+      udisks_notice ("Successfully sent SCSI command SYNCHRONIZE CACHE to %s",
+                     udisks_block_get_device (block));
+    }
+
+  if (!send_scsi_start_stop_unit_command_sync (fd, &error))
     {
       udisks_warning ("Ignoring SCSI command START STOP UNIT failure (%s) on %s",
                       error->message,
@@ -1503,9 +1507,10 @@ handle_power_off (UDisksDrive           *_drive,
     }
   else
     {
-      udisks_notice ("Powering off %s - successfully sent SCSI command START STOP UNIT",
+      udisks_notice ("Successfully sent SCSI command START STOP UNIT to %s",
                      udisks_block_get_device (block));
     }
+
   if (close (fd) != 0)
     {
       g_dbus_method_invocation_return_error (invocation,
