@@ -88,6 +88,8 @@ struct _UDisksLinuxDriveAta
   UDisksThreadedJob *selftest_job;
 
   gboolean     secure_erase_in_progress;
+  unsigned long drive_read, drive_write;
+  gboolean     standby_enabled;
 };
 
 struct _UDisksLinuxDriveAtaClass
@@ -223,6 +225,8 @@ update_pm (UDisksLinuxDriveAta *drive,
   gboolean aam_enabled = FALSE;
   gboolean write_cache_supported = FALSE;
   gboolean write_cache_enabled = FALSE;
+  gboolean read_lookahead_supported = FALSE;
+  gboolean read_lookahead_enabled = FALSE;
   gint aam_vendor_recommended_value = 0;
   guint16 word_82 = 0;
   guint16 word_83 = 0;
@@ -245,8 +249,10 @@ update_pm (UDisksLinuxDriveAta *drive,
   aam_enabled   = word_86 & (1<<9);
   if (aam_supported)
     aam_vendor_recommended_value = (word_94 >> 8);
-  write_cache_supported  = word_82 & (1<<5);
-  write_cache_enabled    = word_85 & (1<<5);
+  write_cache_supported    = word_82 & (1<<5);
+  write_cache_enabled      = word_85 & (1<<5);
+  read_lookahead_supported = word_82 & (1<<6);
+  read_lookahead_enabled   = word_85 & (1<<6);
 
   g_object_freeze_notify (G_OBJECT (drive));
   udisks_drive_ata_set_pm_supported (UDISKS_DRIVE_ATA (drive), !!pm_supported);
@@ -258,6 +264,8 @@ update_pm (UDisksLinuxDriveAta *drive,
   udisks_drive_ata_set_aam_vendor_recommended_value (UDISKS_DRIVE_ATA (drive), aam_vendor_recommended_value);
   udisks_drive_ata_set_write_cache_supported (UDISKS_DRIVE_ATA (drive), !!write_cache_supported);
   udisks_drive_ata_set_write_cache_enabled (UDISKS_DRIVE_ATA (drive), !!write_cache_enabled);
+  udisks_drive_ata_set_read_lookahead_supported (UDISKS_DRIVE_ATA (drive), !!read_lookahead_supported);
+  udisks_drive_ata_set_read_lookahead_enabled (UDISKS_DRIVE_ATA (drive), !!read_lookahead_enabled);
   g_object_thaw_notify (G_OBJECT (drive));
 }
 
@@ -427,6 +435,68 @@ selftest_status_to_string (SkSmartSelfTestExecutionStatus status)
   return ret;
 }
 
+static gboolean get_pm_state (UDisksLinuxDevice *device, GError **error, guchar *count)
+{
+  int fd;
+  gboolean rc = FALSE;
+  /* ATA8: 7.8 CHECK POWER MODE - E5h, Non-Data */
+  UDisksAtaCommandInput input = {.command = 0xe5};
+  UDisksAtaCommandOutput output = {0};
+
+  fd = open (g_udev_device_get_device_file (device->udev_device), O_RDONLY|O_NONBLOCK);
+  if (fd == -1)
+    {
+      g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Error opening device file %s: %m",
+                   g_udev_device_get_device_file (device->udev_device));
+      goto out;
+    }
+
+  if (!udisks_ata_send_command_sync (fd,
+                                     -1,
+                                     UDISKS_ATA_COMMAND_PROTOCOL_NONE,
+                                     &input,
+                                     &output,
+                                     error))
+    {
+      g_prefix_error (error, "Error sending ATA command CHECK POWER MODE: ");
+      goto out;
+    }
+  /* count field is used for the state, see ATA8: table 102 */
+  *count = output.count;
+  rc = TRUE;
+ out:
+  if (fd != -1)
+    close (fd);
+  return rc;
+}
+
+static gboolean update_io_stats (UDisksLinuxDriveAta *drive, UDisksLinuxDevice *device)
+{
+  const gchar *drivepath = g_udev_device_get_sysfs_path (device->udev_device);
+  gchar statpath[PATH_MAX];
+  unsigned long drive_read, drive_write;
+  FILE *statf;
+  gboolean noio = FALSE;
+  snprintf (statpath, sizeof(statpath), "%s/stat", drivepath);
+  statf = fopen (statpath, "r");
+  if (statf == NULL)
+    {
+      udisks_warning ("Failed to open %s\n", statpath);
+    }
+  else
+    {
+      fscanf (statf, "%lu %*u %*u %*u %lu", &drive_read, &drive_write);
+      fclose (statf);
+      noio = drive_read == drive->drive_read && drive_write == drive->drive_write;
+      udisks_debug ("drive_read=%lu, drive_write=%lu, old_drive_read=%lu, old_drive_write=%lu\n",
+                    drive_read, drive_write, drive->drive_read, drive->drive_write);
+      drive->drive_read = drive_read;
+      drive->drive_write = drive_write;
+    }
+  return noio;
+}
+
 /**
  * udisks_linux_drive_ata_refresh_smart_sync:
  * @drive: The #UDisksLinuxDriveAta to refresh.
@@ -519,26 +589,15 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
     }
   else
     {
-      if (sk_disk_open (g_udev_device_get_device_file (device->udev_device), &d) != 0)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "sk_disk_open: %m");
-          goto out;
-        }
-
-      if (sk_disk_check_sleep_mode (d, &awake) != 0)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "sk_disk_check_sleep_mode: %m");
-          goto out;
-        }
-
+      guchar count;
+      gboolean noio = FALSE;
+      if (!get_pm_state(device, error, &count))
+        goto out;
+      awake = count == 0xFF || count == 0x80;
+      if (drive->standby_enabled)
+        noio = update_io_stats (drive, device);
       /* don't wake up disk unless specically asked to */
-      if (nowakeup && !awake)
+      if (nowakeup && (!awake || noio))
         {
           g_set_error (error,
                        UDISKS_ERROR,
@@ -548,6 +607,15 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
         }
     }
 
+  if (sk_disk_open (g_udev_device_get_device_file (device->udev_device), &d) != 0)
+    {
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "sk_disk_open: %m");
+      goto out;
+    }
+
   if (sk_disk_smart_read_data (d) != 0)
     {
       g_set_error (error,
@@ -603,6 +671,8 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
   update_smart (drive, device);
 
   ret = TRUE;
+  /* update stats again to account for the IO we just did to read the SMART info */
+  update_io_stats (drive, device);
 
  out:
   g_clear_object (&device);
@@ -637,7 +707,7 @@ udisks_linux_drive_ata_smart_selftest_sync (UDisksLinuxDriveAta     *drive,
                                             GError                 **error)
 {
   UDisksLinuxDriveObject  *object;
-  UDisksLinuxDevice *device;
+  UDisksLinuxDevice *device = NULL;
   SkDisk *d = NULL;
   gboolean ret = FALSE;
   SkSmartSelfTest test;
@@ -1184,10 +1254,10 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
   UDisksLinuxDriveObject  *object = NULL;
   UDisksDaemon *daemon;
   UDisksLinuxDevice *device = NULL;
-  gint fd = -1;
   GError *error = NULL;
   const gchar *message;
   const gchar *action_id;
+  guchar count;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -1244,38 +1314,12 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
                                              "No udev device");
       goto out;
     }
-
-  fd = open (g_udev_device_get_device_file (device->udev_device), O_RDONLY|O_NONBLOCK);
-  if (fd == -1)
-    {
-      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "Error opening device file %s: %m",
-                                             g_udev_device_get_device_file (device->udev_device));
-      goto out;
-    }
-
-  {
-    /* ATA8: 7.8 CHECK POWER MODE - E5h, Non-Data */
-    UDisksAtaCommandInput input = {.command = 0xe5};
-    UDisksAtaCommandOutput output = {0};
-    if (!udisks_ata_send_command_sync (fd,
-                                       -1,
-                                       UDISKS_ATA_COMMAND_PROTOCOL_NONE,
-                                       &input,
-                                       &output,
-                                       &error))
-      {
-        g_prefix_error (&error, "Error sending ATA command CHECK POWER MODE: ");
-        g_dbus_method_invocation_take_error (invocation, error);
-        goto out;
-      }
-    /* count field is used for the state, see ATA8: table 102 */
-    udisks_drive_ata_complete_pm_get_state (_drive, invocation, output.count);
-  }
+  if (get_pm_state (device, &error, &count))
+    udisks_drive_ata_complete_pm_get_state (_drive, invocation, count);
+  else
+    g_dbus_method_invocation_take_error (invocation, error);
 
  out:
-  if (fd != -1)
-    close (fd);
   g_clear_object (&device);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
@@ -1298,7 +1342,7 @@ handle_pm_standby (UDisksDriveAta        *_drive,
   GError *error = NULL;
   const gchar *message;
   const gchar *action_id;
-  pid_t caller_pid;
+  uid_t caller_uid;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -1331,10 +1375,12 @@ handle_pm_standby (UDisksDriveAta        *_drive,
     }
 
   error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
-                                               &caller_pid,
+                                               &caller_uid,
+                                               NULL,
+                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1354,7 +1400,7 @@ handle_pm_standby (UDisksDriveAta        *_drive,
     {
       action_id = "org.freedesktop.udisks2.ata-standby-system";
     }
-  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
+  else if (!udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
     {
       action_id = "org.freedesktop.udisks2.ata-standby-other-seat";
     }
@@ -1429,8 +1475,8 @@ handle_pm_wakeup (UDisksDriveAta        *_drive,
   GError *error = NULL;
   const gchar *message;
   const gchar *action_id;
-  pid_t caller_pid;
   guchar buf[4096];
+  uid_t caller_uid;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -1463,10 +1509,12 @@ handle_pm_wakeup (UDisksDriveAta        *_drive,
     }
 
   error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
-                                               &caller_pid,
+                                               &caller_uid,
+                                               NULL,
+                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -1486,7 +1534,7 @@ handle_pm_wakeup (UDisksDriveAta        *_drive,
     {
       action_id = "org.freedesktop.udisks2.ata-standby-system";
     }
-  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
+  else if (!udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
     {
       action_id = "org.freedesktop.udisks2.ata-standby-other-seat";
     }
@@ -1592,6 +1640,8 @@ typedef struct
   gint ata_aam_level;
   gboolean ata_write_cache_enabled;
   gboolean ata_write_cache_enabled_set;
+  gboolean ata_read_lookahead_enabled;
+  gboolean ata_read_lookahead_enabled_set;
   UDisksLinuxDriveAta *ata;
   UDisksLinuxDevice *device;
   GVariant *configuration;
@@ -1655,6 +1705,7 @@ apply_configuration_thread_func (gpointer user_data)
           udisks_notice ("Set standby timer to %s (value %d) on %s [%s]",
                          pretty, data->ata_pm_standby, device_file, udisks_drive_get_id (data->drive));
           g_free (pretty);
+          data->ata->standby_enabled = data->ata_pm_standby != 0;
         }
     }
 
@@ -1678,7 +1729,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x (ata_apm_level=%d) to %s: %s (%s, %d)",
-                        input.feature, data->ata_apm_level, device_file,
+                        (guint) input.feature, data->ata_apm_level, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1709,7 +1760,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x (ata_aam_level=%d) to %s: %s (%s, %d)",
-                        input.feature, data->ata_aam_level, device_file,
+                        (guint) input.feature, data->ata_aam_level, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1737,7 +1788,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x to %s: %s (%s, %d)",
-                        input.feature, device_file,
+                        (guint) input.feature, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1749,6 +1800,35 @@ apply_configuration_thread_func (gpointer user_data)
         }
     }
 
+  if (data->ata_read_lookahead_enabled_set)
+    {
+      /* ATA8: 7.48 SET FEATURES - EFh, Non-Data
+       *       7.48.13 Enable/disable read look-ahead
+       */
+      UDisksAtaCommandInput input = {.command = 0xef, .feature = 0x55};
+      UDisksAtaCommandOutput output = {0};
+      if (data->ata_read_lookahead_enabled)
+        input.feature = 0xaa;
+      if (!udisks_ata_send_command_sync (fd,
+                                         -1,
+                                         UDISKS_ATA_COMMAND_PROTOCOL_NONE,
+                                         &input,
+                                         &output,
+                                         &error))
+        {
+          udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x to %s: %s (%s, %d)",
+                        (guint) input.feature, device_file,
+                        error->message, g_quark_to_string (error->domain), error->code);
+          g_clear_error (&error);
+        }
+      else
+        {
+          udisks_notice ("%s Read Look-ahead on %s [%s]",
+                         data->ata_read_lookahead_enabled ? "Enabled" : "Disabled",
+                         device_file, udisks_drive_get_id (data->drive));
+        }
+    }
+
  out:
   if (fd != -1)
     close (fd);
@@ -1779,6 +1859,8 @@ udisks_linux_drive_ata_apply_configuration (UDisksLinuxDriveAta     *drive,
   data->ata_aam_level = -1;
   data->ata_write_cache_enabled = FALSE;
   data->ata_write_cache_enabled_set = FALSE;
+  data->ata_read_lookahead_enabled = FALSE;
+  data->ata_read_lookahead_enabled_set = FALSE;
   data->ata = g_object_ref (drive);
   data->device = g_object_ref (device);
   data->configuration = g_variant_ref (configuration);
@@ -1800,6 +1882,11 @@ udisks_linux_drive_ata_apply_configuration (UDisksLinuxDriveAta     *drive,
       data->ata_write_cache_enabled_set = TRUE;
       has_conf = TRUE;
     }
+  if (g_variant_lookup (configuration, "ata-read-lookahead-enabled", "b", &data->ata_read_lookahead_enabled))
+    {
+      data->ata_read_lookahead_enabled_set = TRUE;
+      has_conf = TRUE;
+    }
 
   /* don't do anything if none of the configuration is set */
   if (!has_conf)
