@@ -33,9 +33,9 @@
 #include "udiskslinuxencrypted.h"
 #include "udiskslinuxblockobject.h"
 #include "udisksdaemon.h"
-#include "udiskspersistentstore.h"
 #include "udisksdaemonutil.h"
-#include "udiskscleanup.h"
+#include "udisksstate.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxencrypted
@@ -238,16 +238,15 @@ handle_unlock (UDisksEncrypted        *encrypted,
   UDisksObject *object = NULL;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   gchar *error_message = NULL;
   gchar *name = NULL;
   gchar *escaped_name = NULL;
   UDisksObject *cleartext_object = NULL;
   UDisksBlock *cleartext_block;
-  GUdevDevice *udev_cleartext_device = NULL;
+  UDisksLinuxDevice *cleartext_device = NULL;
   GError *error = NULL;
   uid_t caller_uid;
-  pid_t caller_pid;
   const gchar *action_id;
   const gchar *message;
   gboolean is_in_crypttab = FALSE;
@@ -266,7 +265,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
    *
@@ -314,18 +313,6 @@ handle_unlock (UDisksEncrypted        *encrypted,
       goto out;
     }
 
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_pid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_error_free (error);
-      goto out;
-    }
-
   /* check if in crypttab file */
   error = NULL;
   if (!check_crypttab (block,
@@ -360,7 +347,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
         {
           action_id = "org.freedesktop.udisks2.encrypted-unlock-system";
         }
-      else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+      else if (!udisks_daemon_util_on_user_seat (daemon, object, caller_uid))
         {
           action_id = "org.freedesktop.udisks2.encrypted-unlock-other-seat";
         }
@@ -438,14 +425,14 @@ handle_unlock (UDisksEncrypted        *encrypted,
                  udisks_block_get_device (block),
                  udisks_block_get_device (cleartext_block));
 
-  udev_cleartext_device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
+  cleartext_device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
 
   /* update the unlocked-luks file */
-  udisks_cleanup_add_unlocked_luks (cleanup,
-                                    udisks_block_get_device_number (cleartext_block),
-                                    udisks_block_get_device_number (block),
-                                    g_udev_device_get_sysfs_attr (udev_cleartext_device, "dm/uuid"),
-                                    caller_uid);
+  udisks_state_add_unlocked_luks (state,
+                                  udisks_block_get_device_number (cleartext_block),
+                                  udisks_block_get_device_number (block),
+                                  g_udev_device_get_sysfs_attr (cleartext_device->udev_device, "dm/uuid"),
+                                  caller_uid);
 
   udisks_encrypted_complete_unlock (encrypted,
                                     invocation,
@@ -459,10 +446,8 @@ handle_unlock (UDisksEncrypted        *encrypted,
   g_free (escaped_name);
   g_free (name);
   g_free (error_message);
-  if (udev_cleartext_device != NULL)
-    g_object_unref (udev_cleartext_device);
-  if (cleartext_object != NULL)
-    g_object_unref (cleartext_object);
+  g_clear_object (&cleartext_device);
+  g_clear_object (&cleartext_object);
   g_clear_object (&object);
 
   return TRUE; /* returning TRUE means that we handled the method invocation */
@@ -479,13 +464,13 @@ handle_lock (UDisksEncrypted        *encrypted,
   UDisksObject *object;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   gchar *error_message;
   gchar *name;
   gchar *escaped_name;
   UDisksObject *cleartext_object;
   UDisksBlock *cleartext_block;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
   uid_t unlocked_by_uid;
   dev_t cleartext_device_from_file;
   GError *error;
@@ -509,7 +494,7 @@ handle_lock (UDisksEncrypted        *encrypted,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
 
   /* TODO: check if the device is mentioned in /etc/crypttab (see crypttab(5)) - if so use that
    *
@@ -547,9 +532,9 @@ handle_lock (UDisksEncrypted        *encrypted,
   cleartext_block = udisks_object_peek_block (cleartext_object);
 
   error = NULL;
-  cleartext_device_from_file = udisks_cleanup_find_unlocked_luks (cleanup,
-                                                                  udisks_block_get_device_number (block),
-                                                                  &unlocked_by_uid);
+  cleartext_device_from_file = udisks_state_find_unlocked_luks (state,
+                                                                udisks_block_get_device_number (block),
+                                                                &unlocked_by_uid);
   if (cleartext_device_from_file == 0)
     {
       /* allow locking stuff not mentioned in unlocked-luks, but treat it like root unlocked it */
@@ -587,7 +572,7 @@ handle_lock (UDisksEncrypted        *encrypted,
     }
 
   device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (cleartext_object));
-  escaped_name = udisks_daemon_util_escape_and_quote (g_udev_device_get_sysfs_attr (device, "dm/name"));
+  escaped_name = udisks_daemon_util_escape_and_quote (g_udev_device_get_sysfs_attr (device->udev_device, "dm/name"));
 
   if (!udisks_daemon_launch_spawned_job_sync (daemon,
                                               object,
@@ -719,7 +704,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
                                               NULL, /* gint *out_status */
                                               &error_message,
                                               passphrases,  /* input_string */
-                                              "cryptsetup luksChangeKey %s",
+                                              "cryptsetup --force-password luksChangeKey %s",
                                               escaped_device))
     {
       g_dbus_method_invocation_return_error (invocation,
