@@ -22,6 +22,7 @@
 #include <glib/gi18n-lib.h>
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
@@ -29,7 +30,9 @@
 #include <stdio.h>
 #include <mntent.h>
 #include <sys/types.h>
+#ifdef HAVE_ACL
 #include <sys/acl.h>
+#endif
 #include <errno.h>
 
 #include <glib/gstdio.h>
@@ -39,10 +42,11 @@
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxfsinfo.h"
 #include "udisksdaemon.h"
-#include "udiskscleanup.h"
+#include "udisksstate.h"
 #include "udisksdaemonutil.h"
 #include "udisksmountmonitor.h"
 #include "udisksmount.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxfilesystem
@@ -77,6 +81,12 @@ static void filesystem_iface_init (UDisksFilesystemIface *iface);
 G_DEFINE_TYPE_WITH_CODE (UDisksLinuxFilesystem, udisks_linux_filesystem, UDISKS_TYPE_FILESYSTEM_SKELETON,
                          G_IMPLEMENT_INTERFACE (UDISKS_TYPE_FILESYSTEM, filesystem_iface_init));
 
+#ifdef HAVE_FHS_MEDIA
+#  define MOUNT_BASE "/media"
+#else
+#  define MOUNT_BASE "/run/media"
+#endif
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
@@ -135,7 +145,7 @@ udisks_linux_filesystem_update (UDisksLinuxFilesystem  *filesystem,
                                 UDisksLinuxBlockObject *object)
 {
   UDisksMountMonitor *mount_monitor;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
   GPtrArray *p;
   GList *mounts;
   GList *l;
@@ -144,7 +154,7 @@ udisks_linux_filesystem_update (UDisksLinuxFilesystem  *filesystem,
   device = udisks_linux_block_object_get_device (object);
 
   p = g_ptr_array_new ();
-  mounts = udisks_mount_monitor_get_mounts_for_dev (mount_monitor, g_udev_device_get_device_number (device));
+  mounts = udisks_mount_monitor_get_mounts_for_dev (mount_monitor, g_udev_device_get_device_number (device->udev_device));
   /* we are guaranteed that the list is sorted so if there are
    * multiple mounts we'll always get the same order
    */
@@ -274,7 +284,7 @@ typedef struct
 
 /* ---------------------- vfat -------------------- */
 
-static const gchar *vfat_defaults[] = { "uid=", "gid=", "shortname=mixed", "dmask=0077", "utf8=1", "showexec", "flush", NULL };
+static const gchar *vfat_defaults[] = { "uid=", "gid=", "shortname=mixed", "utf8=1", "showexec", "flush", NULL };
 static const gchar *vfat_allow[] = { "flush", "utf8=", "shortname=", "umask=", "dmask=", "fmask=", "codepage=", "iocharset=", "usefree", "showexec", NULL };
 static const gchar *vfat_allow_uid_self[] = { "uid=", NULL };
 static const gchar *vfat_allow_gid_self[] = { "gid=", NULL };
@@ -282,7 +292,7 @@ static const gchar *vfat_allow_gid_self[] = { "gid=", NULL };
 /* ---------------------- ntfs -------------------- */
 /* this is assuming that ntfs-3g is used */
 
-static const gchar *ntfs_defaults[] = { "uid=", "gid=", "dmask=0077", "fmask=0177", NULL };
+static const gchar *ntfs_defaults[] = { "uid=", "gid=", NULL };
 static const gchar *ntfs_allow[] = { "umask=", "dmask=", "fmask=", "locale=", "norecover", "ignore_case", "windows_names", "compression", "nocompression", NULL };
 static const gchar *ntfs_allow_uid_self[] = { "uid=", NULL };
 static const gchar *ntfs_allow_gid_self[] = { "gid=", NULL };
@@ -296,11 +306,18 @@ static const gchar *iso9660_allow_gid_self[] = { "gid=", NULL };
 
 /* ---------------------- udf -------------------- */
 
-static const gchar *udf_defaults[] = { "uid=", "gid=", "iocharset=utf8", "umask=0077", NULL };
+static const gchar *udf_defaults[] = { "uid=", "gid=", "iocharset=utf8", NULL };
 static const gchar *udf_allow[] = { "iocharset=", "umask=", NULL };
 static const gchar *udf_allow_uid_self[] = { "uid=", NULL };
 static const gchar *udf_allow_gid_self[] = { "gid=", NULL };
 
+/* ---------------------- exfat -------------------- */
+
+static const gchar *exfat_defaults[] = { "uid=", "gid=", "iocharset=utf8", "namecase=0", "errors=remount-ro", "umask=0077", NULL };
+static const gchar *exfat_allow[] = { "dmask=", "errors=", "fmask=", "iocharset=", "namecase=", "umask=", NULL };
+static const gchar *exfat_allow_uid_self[] = { "uid=", NULL };
+static const gchar *exfat_allow_gid_self[] = { "gid=", NULL };
+
 /* ------------------------------------------------ */
 /* TODO: support context= */
 
@@ -312,6 +329,7 @@ static const FSMountOptions fs_mount_options[] =
     { "ntfs", ntfs_defaults, ntfs_allow, ntfs_allow_uid_self, ntfs_allow_gid_self },
     { "iso9660", iso9660_defaults, iso9660_allow, iso9660_allow_uid_self, iso9660_allow_gid_self },
     { "udf", udf_defaults, udf_allow, udf_allow_uid_self, udf_allow_gid_self },
+    { "exfat", exfat_defaults, exfat_allow, exfat_allow_uid_self, exfat_allow_gid_self },
   };
 
 /* ------------------------------------------------ */
@@ -339,15 +357,18 @@ find_mount_options_for_fs (const gchar *fstype)
 static gid_t
 find_primary_gid (uid_t uid)
 {
-  struct passwd *pw;
+  struct passwd *pw = NULL;
+  struct passwd pwstruct;
+  gchar pwbuf[8192];
+  int rc;
   gid_t gid;
 
   gid = (gid_t) - 1;
 
-  pw = getpwuid (uid);
-  if (pw == NULL)
+  rc = getpwuid_r (uid, &pwstruct, pwbuf, sizeof pwbuf, &pw);
+  if (rc != 0 || pw == NULL)
     {
-      udisks_warning ("Error looking up uid %d: %m", uid);
+      udisks_warning ("Error looking up uid %u: %m", uid);
       goto out;
     }
   gid = pw->pw_gid;
@@ -361,7 +382,10 @@ is_uid_in_gid (uid_t uid,
                gid_t gid)
 {
   gboolean ret;
-  struct passwd *pw;
+  struct passwd *pw = NULL;
+  struct passwd pwstruct;
+  gchar pwbuf[8192];
+  int rc;
   static gid_t supplementary_groups[128];
   int num_supplementary_groups = 128;
   int n;
@@ -370,10 +394,10 @@ is_uid_in_gid (uid_t uid,
 
   ret = FALSE;
 
-  pw = getpwuid (uid);
-  if (pw == NULL)
+  rc = getpwuid_r (uid, &pwstruct, pwbuf, sizeof pwbuf, &pw);
+  if (rc != 0 || pw == NULL)
     {
-      udisks_warning ("Error looking up uid %d: %m", uid);
+      udisks_warning ("Error looking up uid %u: %m", uid);
       goto out;
     }
   if (pw->pw_gid == gid)
@@ -384,7 +408,7 @@ is_uid_in_gid (uid_t uid,
 
   if (getgrouplist (pw->pw_name, pw->pw_gid, supplementary_groups, &num_supplementary_groups) < 0)
     {
-      udisks_warning ("Error getting supplementary groups for uid %d: %m", uid);
+      udisks_warning ("Error getting supplementary groups for uid %u: %m", uid);
       goto out;
     }
 
@@ -532,7 +556,7 @@ prepend_default_mount_options (const FSMountOptions *fsmo,
 
           if (strcmp (option, "uid=") == 0)
             {
-              s = g_strdup_printf ("uid=%d", caller_uid);
+              s = g_strdup_printf ("uid=%u", caller_uid);
               g_ptr_array_add (options, s);
             }
           else if (strcmp (option, "gid=") == 0)
@@ -540,7 +564,7 @@ prepend_default_mount_options (const FSMountOptions *fsmo,
               gid = find_primary_gid (caller_uid);
               if (gid != (gid_t) - 1)
                 {
-                  s = g_strdup_printf ("gid=%d", gid);
+                  s = g_strdup_printf ("gid=%u", gid);
                   g_ptr_array_add (options, s);
                 }
             }
@@ -781,6 +805,7 @@ ensure_utf8 (const gchar *s)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+#ifdef HAVE_ACL
 static gboolean
 add_acl (const gchar  *path,
          uid_t         uid,
@@ -801,21 +826,19 @@ add_acl (const gchar  *path,
       acl_calc_mask (&acl) == -1 ||
       acl_set_file (path, ACL_TYPE_ACCESS, acl) == -1)
     {
-      g_set_error (error,
-                   G_IO_ERROR,
-                   g_io_error_from_errno (errno),
+      udisks_warning(
                    "Adding read ACL for uid %d to `%s' failed: %m",
                    (gint) uid, path);
-      goto out;
+      chown(path, uid, -1);
     }
 
   ret = TRUE;
 
- out:
   if (acl != NULL)
     acl_free (acl);
   return ret;
 }
+#endif
 
 /*
  * calculate_mount_point: <internal>
@@ -840,6 +863,8 @@ calculate_mount_point (UDisksDaemon              *daemon,
                        const gchar               *fs_type,
                        GError                   **error)
 {
+  UDisksLinuxBlockObject *object = NULL;
+  gboolean fs_shared = FALSE;
   const gchar *label = NULL;
   const gchar *uuid = NULL;
   gchar *escaped_user_name = NULL;
@@ -858,28 +883,44 @@ calculate_mount_point (UDisksDaemon              *daemon,
       uuid = udisks_block_get_id_uuid (block);
     }
 
-  /* If we know the user-name and it doesn't have any '/' character in
-   * it, mount in /run/media/$USER
-   */
-  if (user_name != NULL && strstr (user_name, "/") == NULL)
+  object = udisks_daemon_util_dup_object (block, NULL);
+  if (object != NULL)
     {
-      mount_dir = g_strdup_printf ("/run/media/%s", user_name);
+      UDisksLinuxDevice *device = udisks_linux_block_object_get_device (object);
+      if (device != NULL)
+        {
+          if (device->udev_device != NULL)
+            {
+              /* TODO: maybe introduce Block:HintFilesystemShared instead of pulling it directly from the udev device */
+              fs_shared = g_udev_device_get_property_as_boolean (device->udev_device, "UDISKS_FILESYSTEM_SHARED");
+            }
+          g_object_unref (device);
+        }
+    }
+
+  /* If we know the user-name and it doesn't have any '/' character in
+   * it, mount in MOUNT_BASE/$USER
+   */
+  if (!fs_shared && (user_name != NULL && strstr (user_name, "/") == NULL))
+    {
+      mount_dir = g_strdup_printf (MOUNT_BASE "/%s", user_name);
       if (!g_file_test (mount_dir, G_FILE_TEST_EXISTS))
         {
-          /* First ensure that /run/media exists */
-          if (!g_file_test ("/run/media", G_FILE_TEST_EXISTS))
+          /* First ensure that MOUNT_BASE exists */
+          if (g_mkdir (MOUNT_BASE, 0755) != 0 && errno != EEXIST)
             {
-              if (g_mkdir ("/run/media", 0755) != 0)
-                {
-                  g_set_error (error,
-                               UDISKS_ERROR,
-                               UDISKS_ERROR_FAILED,
-                               "Error creating directory /run/media: %m");
-                  goto out;
-                }
+              g_set_error (error,
+                           UDISKS_ERROR,
+                           UDISKS_ERROR_FAILED,
+                           "Error creating directory " MOUNT_BASE ": %m");
+              goto out;
             }
-          /* Then create the per-user /run/media/$USER */
-          if (g_mkdir (mount_dir, 0700) != 0)
+          /* Then create the per-user MOUNT_BASE/$USER */
+#ifdef HAVE_ACL
+          if (g_mkdir (mount_dir, 0700) != 0 && errno != EEXIST)
+#else
+          if (g_mkdir (mount_dir, 0750) != 0 && errno != EEXIST)
+#endif
             {
               g_set_error (error,
                            UDISKS_ERROR,
@@ -889,8 +930,17 @@ calculate_mount_point (UDisksDaemon              *daemon,
               goto out;
             }
           /* Finally, add the read+execute ACL for $USER */
+#ifdef HAVE_ACL
           if (!add_acl (mount_dir, uid, error))
             {
+#else
+          if (chown (mount_dir, -1, gid) == -1)
+            {
+               g_set_error (error, G_IO_ERROR,
+                            g_io_error_from_errno (errno),
+                            "Failed to change gid to %d for %s: %m",
+                            (gint) gid, mount_dir);
+#endif
               if (rmdir (mount_dir) != 0)
                 udisks_warning ("Error calling rmdir() on %s: %m", mount_dir);
               goto out;
@@ -955,13 +1005,14 @@ calculate_mount_point (UDisksDaemon              *daemon,
       else
         {
           g_free (mount_point);
-          mount_point = g_strdup_printf ("%s%d", orig_mount_point, n++);
+          mount_point = g_strdup_printf ("%s%u", orig_mount_point, n++);
         }
     }
   g_free (orig_mount_point);
   g_free (mount_dir);
 
  out:
+  g_clear_object (&object);
   g_free (escaped_user_name);
   return mount_point;
 }
@@ -1025,6 +1076,14 @@ is_in_fstab (UDisksBlock        *block,
         {
           device = g_strdup_printf ("/dev/disk/by-label/%s", m->mnt_fsname + 6);
         }
+      else if (g_str_has_prefix (m->mnt_fsname, "PARTUUID="))
+        {
+          device = g_strdup_printf ("/dev/disk/by-partuuid/%s", m->mnt_fsname + 9);
+        }
+      else if (g_str_has_prefix (m->mnt_fsname, "PARTLABEL="))
+        {
+          device = g_strdup_printf ("/dev/disk/by-partlabel/%s", m->mnt_fsname + 10);
+        }
       else if (g_str_has_prefix (m->mnt_fsname, "/dev"))
         {
           device = g_strdup (m->mnt_fsname);
@@ -1102,10 +1161,9 @@ handle_mount (UDisksFilesystem       *filesystem,
   UDisksObject *object;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   uid_t caller_uid;
   gid_t caller_gid;
-  pid_t caller_pid;
   const gchar * const *existing_mount_points;
   const gchar *probed_fs_usage;
   gchar *fs_type_to_use;
@@ -1148,7 +1206,7 @@ handle_mount (UDisksFilesystem       *filesystem,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
 
   /* check if mount point is managed by e.g. /etc/fstab or similar */
   if (is_system_managed (block, &mount_point_to_use, &fstab_mount_options))
@@ -1193,18 +1251,6 @@ handle_mount (UDisksFilesystem       *filesystem,
       goto out;
     }
 
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_pid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_error_free (error);
-      goto out;
-    }
-
   if (system_managed)
     {
       gint status;
@@ -1226,7 +1272,7 @@ handle_mount (UDisksFilesystem       *filesystem,
                 {
                   action_id = "org.freedesktop.udisks2.filesystem-mount-system";
                 }
-              else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+              else if (!udisks_daemon_util_on_user_seat (daemon, object, caller_uid))
                 {
                   action_id = "org.freedesktop.udisks2.filesystem-mount-other-seat";
                 }
@@ -1303,17 +1349,17 @@ handle_mount (UDisksFilesystem       *filesystem,
                                                  error_message);
           goto out;
         }
-      udisks_notice ("Mounted %s (system) at %s on behalf of uid %d",
+      udisks_notice ("Mounted %s (system) at %s on behalf of uid %u",
                      udisks_block_get_device (block),
                      mount_point_to_use,
                      caller_uid);
 
       /* update the mounted-fs file */
-      udisks_cleanup_add_mounted_fs (cleanup,
-                                     mount_point_to_use,
-                                     udisks_block_get_device_number (block),
-                                     caller_uid,
-                                     TRUE); /* fstab_mounted */
+      udisks_state_add_mounted_fs (state,
+                                   mount_point_to_use,
+                                   udisks_block_get_device_number (block),
+                                   caller_uid,
+                                   TRUE); /* fstab_mounted */
 
       udisks_filesystem_complete_mount (filesystem, invocation, mount_point_to_use);
       goto out;
@@ -1388,7 +1434,7 @@ handle_mount (UDisksFilesystem       *filesystem,
         {
           action_id = "org.freedesktop.udisks2.filesystem-mount-system";
         }
-      else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+      else if (!udisks_daemon_util_on_user_seat (daemon, object, caller_uid))
         {
           action_id = "org.freedesktop.udisks2.filesystem-mount-other-seat";
         }
@@ -1464,13 +1510,13 @@ handle_mount (UDisksFilesystem       *filesystem,
     }
 
   /* update the mounted-fs file */
-  udisks_cleanup_add_mounted_fs (cleanup,
-                                 mount_point_to_use,
-                                 udisks_block_get_device_number (block),
-                                 caller_uid,
-                                 FALSE); /* fstab_mounted */
+  udisks_state_add_mounted_fs (state,
+                               mount_point_to_use,
+                               udisks_block_get_device_number (block),
+                               caller_uid,
+                               FALSE); /* fstab_mounted */
 
-  udisks_notice ("Mounted %s at %s on behalf of uid %d",
+  udisks_notice ("Mounted %s at %s on behalf of uid %u",
                  udisks_block_get_device (block),
                  mount_point_to_use,
                  caller_uid);
@@ -1518,14 +1564,14 @@ handle_unmount (UDisksFilesystem       *filesystem,
   UDisksObject *object;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   gchar *mount_point;
   gchar *fstab_mount_options;
   gchar *escaped_mount_point;
   GError *error;
   uid_t mounted_by_uid;
   uid_t caller_uid;
-  gint status;
+  gint status = 0;
   gchar *error_message;
   const gchar *const *mount_points;
   gboolean opt_force;
@@ -1553,7 +1599,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
   system_managed = FALSE;
 
   if (options != NULL)
@@ -1650,7 +1696,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
                                                  error_message);
           goto out;
         }
-      udisks_notice ("Unmounted %s (system) from %s on behalf of uid %d",
+      udisks_notice ("Unmounted %s (system) from %s on behalf of uid %u",
                      udisks_block_get_device (block),
                      mount_point,
                      caller_uid);
@@ -1659,10 +1705,10 @@ handle_unmount (UDisksFilesystem       *filesystem,
     }
 
   error = NULL;
-  mount_point = udisks_cleanup_find_mounted_fs (cleanup,
-                                                udisks_block_get_device_number (block),
-                                                &mounted_by_uid,
-                                                &fstab_mounted);
+  mount_point = udisks_state_find_mounted_fs (state,
+                                              udisks_block_get_device_number (block),
+                                              &mounted_by_uid,
+                                              &fstab_mounted);
   if (mount_point == NULL)
     {
       /* allow unmounting stuff not mentioned in mounted-fs, but treat it like root mounted it */
@@ -1740,9 +1786,9 @@ handle_unmount (UDisksFilesystem       *filesystem,
       goto out;
     }
 
-  /* OK, filesystem unmounted.. the cleanup routines will remove the mountpoint for us */
+  /* OK, filesystem unmounted.. the state/cleanup routines will remove the mountpoint for us */
 
-  udisks_notice ("Unmounted %s on behalf of uid %d",
+  udisks_notice ("Unmounted %s on behalf of uid %u",
                  udisks_block_get_device (block),
                  caller_uid);
 
@@ -1800,9 +1846,9 @@ handle_set_label (UDisksFilesystem       *filesystem,
   UDisksBaseJob *job;
   const gchar *action_id;
   const gchar *message;
+  gchar *real_label = NULL;
   uid_t caller_uid;
   gid_t caller_gid;
-  pid_t caller_pid;
   gchar *command;
   gchar *tmp;
   GError *error;
@@ -1822,18 +1868,6 @@ handle_set_label (UDisksFilesystem       *filesystem,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   block = udisks_object_peek_block (object);
 
-  error = NULL;
-  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
-                                               invocation,
-                                               NULL /* GCancellable */,
-                                               &caller_pid,
-                                               &error))
-    {
-      g_dbus_method_invocation_return_gerror (invocation, error);
-      g_error_free (error);
-      goto out;
-    }
-
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
@@ -1874,8 +1908,9 @@ handle_set_label (UDisksFilesystem       *filesystem,
       goto out;
     }
 
-  /* VFAT does not allow some characters; as mlabel hangs with interactive
-   * question in this case, check in advance */
+  /* VFAT does not allow some characters; as dosfslabel does not enforce this,
+   * check in advance; also, VFAT only knows upper-case characters, dosfslabel
+   * enforces this */
   if (g_strcmp0 (probed_fs_type, "vfat") == 0)
     {
       const gchar *forbidden = "\"*/:<>?\\|";
@@ -1892,6 +1927,11 @@ handle_set_label (UDisksFilesystem       *filesystem,
                goto out;
             }
         }
+
+      /* we need to remember that we make a copy, so assign it to a new
+       * variable, too */
+      real_label = g_ascii_strup (label, -1);
+      label = real_label;
     }
 
   /* Fail if the device is already mounted and the tools/drivers doesn't
@@ -1921,13 +1961,16 @@ handle_set_label (UDisksFilesystem       *filesystem,
    * will be replaced by the name of the drive/device in question
    */
   message = N_("Authentication is required to change the filesystem label on $(drive)");
-  if (udisks_block_get_hint_system (block))
+  if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
     {
-      action_id = "org.freedesktop.udisks2.modify-device-system";
-    }
-  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
-    {
-      action_id = "org.freedesktop.udisks2.modify-device-other-seat";
+      if (udisks_block_get_hint_system (block))
+        {
+          action_id = "org.freedesktop.udisks2.modify-device-system";
+        }
+      else if (!udisks_daemon_util_on_user_seat (daemon, UDISKS_OBJECT (object), caller_uid))
+        {
+          action_id = "org.freedesktop.udisks2.modify-device-other-seat";
+        }
     }
 
   /* Check that the user is actually authorized to change the
@@ -1966,6 +2009,8 @@ handle_set_label (UDisksFilesystem       *filesystem,
                     invocation);
 
  out:
+  /* for some FSes we need to copy and modify label; free our copy */
+  g_free (real_label);
   g_free (command);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
